#version 330

uniform sampler2D colorTexture;

uniform sampler2D specularTexture;
uniform bool hasSpecular;
uniform int shininess;

uniform sampler2D normalTexture;
uniform bool hasNormal;

//uniform vec3 dir_light_col;
uniform float dir_light_int; //ambient light strength

in vec2 texCoord;
in vec3 N; //normal
in vec3 V; //view
in vec3 L;

in vec3 T; //tangent
in vec3 B; //bitangent

out vec3 FragColor;

void main()
{
	vec3 Normal;
	//Normal
	if (hasNormal){
		//Read Normal from Texture
		vec3 tex_Normal = normalize(texture2D( normalTexture, texCoord).rgb*2.0 - 1.0); //in tangent Space

		//Transform to Worldspace
		mat3 TBN = mat3(normalize(T), normalize(B), normalize(N));
		Normal = TBN * tex_Normal;

	} else {
		Normal = N;
	}


	//Diffuse Part
	vec3 tex_col = vec3(texture2D(colorTexture, texCoord));

	vec3 H = normalize(normalize(V)+normalize(L));

	float cosNL = max(dot(normalize(Normal), normalize(L)), 0.0);

	//float factor = cosNL*(1-dir_light_int)+dir_light_int;
	float factor = cosNL*(0.8)+dir_light_int;

	tex_col = vec3(tex_col.r*factor, tex_col.g*factor, tex_col.b*factor); 

	//Specular Part
	vec3 spec_col;
	if (hasSpecular) {
		spec_col = vec3(texture2D(specularTexture, texCoord));
	} else {
		spec_col = vec3(1,1,1);
	}
	spec_col = spec_col * cosNL* pow(max(dot(normalize(Normal), H), 0),shininess);

	FragColor = tex_col+spec_col;
}