#version 330

uniform sampler2D colorTexture;
uniform sampler2D colorTexture2;

uniform sampler2D specularTexture;
uniform sampler2D specularTexture2;

//uniform bool hasSpecular;
uniform int shininess;

uniform sampler2D normalTexture;
uniform sampler2D normalTexture2;
//uniform bool hasNormal;

//uniform vec3 dir_light_col;
uniform float dir_light_int; //ambient light strength

in vec2 texCoord;
in vec3 N; //normal
in vec3 V; //view
in vec3 L;

in vec3 T; //tangent
in vec3 B; //bitangent

in float height; //World-Y (Height value)

//in float angle; //cos to upvector

out vec3 FragColor;

void main()
{

float angle = dot(normalize(N), vec3(0,1,0)); //angle to Upvector

float upperlimit = 0.9f;
float lowerlimit = 0.7f;
float terrainheight = 400.0f;

float hf = (height/terrainheight)*0.5; //Heightfactor

angle -=hf; //modify the angle to be more likely be rocks on high terrain points

//Calculate angle

vec3 tex_Normal;
vec3 tex_Normal1 = normalize(texture2D( normalTexture, texCoord).rgb*2.0 - 1.0); //in tangent Space
vec3 tex_col1 = vec3(texture2D(colorTexture, texCoord));
vec3 spec_col1= vec3(texture2D(specularTexture, texCoord));
vec3 tex_Normal2 = normalize(texture2D( normalTexture2, texCoord).rgb*2.0 - 1.0); //in tangent Space
vec3 tex_col2 = vec3(texture2D(colorTexture2, texCoord));
vec3 spec_col2 = vec3(texture2D(specularTexture2, texCoord));
vec3 tex_col;
vec3 spec_col;
if (angle >= upperlimit) {
	tex_Normal = tex_Normal1;
	tex_col = tex_col1;
	spec_col= spec_col1;
} else if (angle < lowerlimit) {
	//Read Normal from Texture
	tex_Normal = tex_Normal2;
	tex_col = tex_col2;
	spec_col= spec_col2;
} else {
	//interpolate
	float f1 = (angle-lowerlimit)*(1/(upperlimit-lowerlimit)); //stretch value from [lowerlimit,upperlimit] to [0,1]

	tex_Normal = mix(tex_Normal2, tex_Normal1, f1);
	tex_col = mix(tex_col2,tex_col1,f1);
	spec_col = mix(spec_col2, spec_col1, f1);
}

		

		//Transform to Worldspace
		mat3 TBN = mat3(normalize(T), normalize(B), normalize(N));
		vec3 Normal = TBN * tex_Normal;

	

	vec3 H = normalize(normalize(V)+normalize(L));

	float cosNL = max(dot(normalize(Normal), normalize(L)), 0.0);

	//float factor = cosNL*(1-dir_light_int)+dir_light_int;
	float factor = cosNL*(0.8)+dir_light_int;

	tex_col = vec3(tex_col.r*factor, tex_col.g*factor, tex_col.b*factor); 

	//Specular Part
	

	spec_col = spec_col * cosNL* pow(max(dot(normalize(Normal), H), 0),shininess);

	FragColor = tex_col+spec_col;
}